class DimensionChangerService {

    getWidgetsItemsForMenu(dcWidget) {
        let menuWidgets = [];
        let dashboardWidgets = this.getDashboardWidgets();
        let selectedWidgets = this.getSelectedWidgets(dcWidget);

        _.forEach(dashboardWidgets, (widget) => {

            let selectedWidgetIndex = _.findIndex(selectedWidgets, (element) => {
                return element.id === widget.oid;
            });

            if (!~selectedWidgetIndex && (~widget.type.indexOf('pivot') || ~widget.type.indexOf('chart'))) {
                menuWidgets.push({
                    id: widget.oid,
                    selectionData: {
                        id: widget.oid,
                        title: (widget.title || 'Untitled') + ' ( ' + widget.type + ' ) ',
                        panelToChange: '',
                        index: 0
                    },
                    type: 'option',
                    caption: (widget.title || 'Untitled') + ' ( ' + widget.type + ' ) ',
                    desc: (widget.title || 'Untitled'),
                    classes: 'mi-caption',
                    closing: true,
                    xLarge: true
                });

                menuWidgets.push({
                    type: 'separator'
                });
            }
        });

        if (!menuWidgets.length) {
            menuWidgets.push({
                type: 'nonClickableNonSelectable',
                caption: 'Not available',
                clasees: 'mi-caption',
                xLarge: true
            })
        }

        return menuWidgets;
    }

    getSelectedWidgets(dcWidget) {
        let selectedWidgets = dcWidget.style.selectedWidgets;

        if (selectedWidgets && selectedWidgets.length) {
            this.filterRemovedWidgetAndUpdateTitles(selectedWidgets);
        }

        return selectedWidgets;
    }

    filterRemovedWidgetAndUpdateTitles(selectedWidgets) {
        let dashboardWidgets = this.getDashboardWidgets();

        selectedWidgets.map((item, index) => {
            let widget = _.find(dashboardWidgets, dWidget => item.id === dWidget.oid);

            if (!widget) {
                selectedWidgets.splice(index, 1);
            } else {
                item.title = (widget.title || 'Untitled') + ' ( ' + widget.type + ' ) '
            }

            return item;

        });
    }

    removeWidget(dcWidget, index) {
        dcWidget.style.selectedWidgets.splice(index, 1);
        return this.getSelectedWidgets(dcWidget);
    }

    addWidget(dcWidget, widget) {
        dcWidget.style.selectedWidgets.push(widget.selectionData);
        return this.getSelectedWidgets(dcWidget);
    }

    updateWidget(dcWidget, widget, index) {
        dcWidget.style.selectedWidgets[index] = widget;
        return this.getSelectedWidgets(dcWidget);
    }

    //get all dashboard widgets
    getDashboardWidgets() {
        return $$get(prism, 'activeDashboard.widgets.$$widgets');
    }

    // setWidgetsToApplyOn to DimensionChanger Widget
    setSelectedWidgets(dcWidget, widgets) {
        $$set(dcWidget, 'style.selectedWidgets', widgets);
    }
}

mod.service('dimensionChangerService', [
    DimensionChangerService
]);
