/**
 * @fileOverview    Dimensions Changer
 * @author          Dana Raijman
 * @version         V1.0
 * @SiSenseVersion  V5.7.6
 **/

prism.run([

    function () {
        prism.registerWidget("DimensionChanger", {
            name : "DimensionChanger",
            family : "DimensionChanger",
            title : "Dimension Changer",
            iconSmall : "/plugins/DimensionChanger/styles/dimension.png",
            styleEditorTemplate : '/plugins/DimensionChanger/views/styler.html',
            style : {},
            sizing: {// sizing must be stated
                minHeight: 32, //header
                maxHeight: 2048,
                minWidth: 128,
                maxWidth: 2048,
                height:32,
                defaultWidth: 512
            },
            data : {
                selection : [],
                defaultQueryResult : {},
                panels : [
                    {//dimension panel
                        name : 'rows',
                        type : 'visible',
                        metadata : {
                            types : ['dimensions'],
                            maxitems : -1
                        }
                    }, {//measure panel
                        name : 'values',
                        type : 'visible',
                        metadata : {

                            types : ['measures'],
                            maxitems : -1
                        }
                    },
                    {
                        name: 'filters',
                        type: 'filters',
                        metadata: {

                            types: ['dimensions'],
                            maxitems: -1
                        }
                    }
                ],
                allocatePanel : function (widget, metadataItem) {
                    if (prism.$jaql.isMeasure(metadataItem) && widget.metadata.panel("values").items.length === 0) {
                        return "values";
                    }
                    // item
                    else if (!prism.$jaql.isMeasure(metadataItem) && widget.metadata.panel("rows").items.length < 3) {
                        return "rows";
                    }
                },
                // returns true/ reason why the given item configuration is not/supported by the widget
                isSupported : function (items) {
                    return this.rankMetadata(items, null, null) > -1;
                },
                // ranks the compatibility of the given metadata items with the widget
                rankMetadata : function (items, type, subtype) {
                    var a = prism.$jaql.analyze(items);

                    // require at least 2 dimensions of lat and lng and 1 measure
                    if (a.dimensions.length >= 1 || a.measures.length >= 1) {
                        return 0;
                    }

                    return -1;
                },
                // populates the metadata items to the widget
                populateMetadata : function (widget, items) {
                    var a = prism.$jaql.analyze(items);

                    // allocating dimensions
                    widget.metadata.panel("rows").push(a.dimensions);
                    widget.metadata.panel("values").push(a.measures);
                    widget.metadata.panel("filters").push(a.filters);
                },
                // builds a jaql query from the given widget
                buildQuery : function (widget) {
                    var query = {
                        datasource : widget.datasource,
                        metadata : []
                    };

                    // pushing non-KPI location items
                    widget.metadata.panel("rows").items.forEach(function (item) {
                        query.metadata.push(item);
                    });

                    // pushing KPI items
                    widget.metadata.panel("values").items.forEach(function(item) {
                        query.metadata.push(item)
                    });

                    // filters
                    widget.metadata.panel("filters").items.forEach(function (item) {
                        item = $$.object.clone(item, true);
                        item.panel = "scope";
                        query.metadata.push(item);
                    });

                    return query;
                },
                // prepares the widget-specific query result from the given result data-table
                processResult : function (widget, queryResult) {
                    return queryResult;
                }
            },
            render : function (widget, e) {
                var style = $$get(widget, 'style');
                var widgetIds = (style.selectedWidgets && style.selectedWidgets.length) ? style.selectedWidgets : (e.widgetIds || []);

                var defaultSelectedColor = '#86b817';
                var defaultFontColor = '#cccccc';
                var defaultDescColor = '#a5a5a5';

                var selectedColor  = style.selectedColor || e.selectedColor || defaultSelectedColor;
                var fontColor  = style.fontColor || e.fontColor || defaultFontColor;
                var descColor = style.descColor || style.descColor || defaultDescColor;

                var prefixText = style.prefixText || e.prefixText || '';
                var suffixText = style.suffixText || e.suffixText || '';
                var parentWidth = style.width || e.elementWidth || '103%';
                var height = e.height || 32;

                var dimensions;
                var lastSelected;

                if (e.dimensions) {
                    dimensions = e.dimensions
                } else if (widget.rawQueryResult) {
                    dimensions = _.map(widget.rawQueryResult.metadata, function(md){ return md.jaql });
                } else {
                    return;
                }

                function isBlank(str) {
                    return (!str || /^\s*$/.test(str));
                }

                var desc = prefixText + ' ' + (widget.desc ? widget.desc : '');
                var $desc = $('<span class="descDefaultCSS" >' + desc + '</span>').css({'color': descColor, 'margin-right': '15px'});
                var dimensionList = $('<div class="listDefaultCSS"></div>');

                if(desc && !isBlank(desc)) {
                    dimensionList.append($desc);
                }

                for (var i =0; i < dimensions.length; i++) {
                    var $listItem = $('<span class="listItemDefaultCSS">' + dimensions[i].title + '</span>');

                    $listItem.attr('tag',dimensions[i].title);

                    if(dimensions[i].default){
                        select($listItem);
                    }

                    if (widget.lastSelected === dimensions[i].title) {
                        select($listItem);
                    }

                    dimensionList.append($listItem);

                    var $vSeparator = $('<div class="vSeparatorCSS"></div>');

                    (i !== dimensions.length - 1) && dimensionList.append($vSeparator);
                }

                if (suffixText.length) {
                    var $suffixText = $('<span class="descDefaultCSS" >' + suffixText + '</span>').css({'color': descColor, 'margin-left': '15px'});

                    dimensionList.append($suffixText)
                }

                var $lmnt = $(e.element);

                $lmnt.parent().addClass('parentCSS');
                $lmnt.empty().addClass('defaultCSS').append(dimensionList);

                $lmnt.css('height', height);
                $lmnt.css('color', fontColor);
                $lmnt.css('width', parentWidth);

                function dimensionClicked() {

                    if (prism.$ngscope.appstate === 'widget') {
                        return;
                    }

                    select(this);
                    changeDimension(this);
                }

                function changeDimension(element) {
                    var dim = getDimensionsByCaption($(element).attr('tag'));

                    if (!dim) {
                        return;
                    }

                    if (!widgetIds) {
                        return;
                    }

                    for(var i = widgetIds.length-1; i>=0; i--) {
                        var widgetId = widgetIds[i].id;
                        var widgetToApplyOn = getWidgetById(widgetId);

                        if(widgetToApplyOn) {
                            var item = widgetToApplyOn.metadata.panel(widgetIds[i].panelToChange.toLowerCase()).items[widgetIds[i].index] ;

                            if (item) {
                                item.jaql = dim;
                                widgetToApplyOn.changesMade();
                                widgetToApplyOn.refresh();
                            } else {
                                console.warn('Couldn\'t switch dimension. Please, review your target widget panel items');
                            }

                        }
                    }
                }

                function getDimensionsByCaption(title) {
                    for(var i = dimensions.length-1; i >=0; i--){
                        if(dimensions[i].title === title){
                            return dimensions[i];
                        }
                    }
                }

                function getWidgetById(widgetId) {
                    var widgets = $$get(widget,'dashboard.widgets.$$widgets');

                    if(!widgets){
                        console.log('unable to get widgets from dashboard');
                        return;
                    }

                    for(var i = widgets.length-1; i >=0; i--) {
                        if(widgets[i].oid === widgetId){
                            return widgets[i];
                        }
                    }
                }

                function select(item) {

                    if (lastSelected) {
                        $(lastSelected).css('color',fontColor)
                    }
                    lastSelected = $(item);

                    $(item).css('color',selectedColor);

                    widget.lastSelected = $(item).attr('tag');
                    widget.changesMade();
                }

                function addEvents(dimList) {
                    for (var i = dimList.length -1; i >= 0; i--) {
                        $('widget[widgetid="' + widget.oid + '"] span[tag="'+ dimList[i].title+'"]').on('click',dimensionClicked)
                    }
                }

                addEvents(dimensions);
            },
            destroy : function (e) {}
        });

    }
])


